///////////////////////////////////////////////////////
// Fideltronik R&D Department, Copyright (c)2003
//         $Author: ltokarski $
//         $Date: 2004/07/20 10:18:55 $
//         $Locker:  $
//         $Revision: 1.13 $
///////////////////////////////////////////////////////
//	Opis funkcjonalny moduu:
//          Wsplna obsuga rnych modeli UPSw
///////////////////////////////////////////////////////
// Historia zmian:
// $Log: ups.cpp,v $
// Revision 1.13  2004/07/20 10:18:55  ltokarski
// Z0013 - wypisywanie w statusie protokolu komunikacji
//
// Revision 1.12  2004/07/09 11:32:01  ltokarski
// Z0012 - zapis biezacego statusu do pliku
//
// Revision 1.11  2004/06/29 10:19:25  ltokarski
// Z0008 - wejscie w tlo po zainicjowaniu komunikacji i zwracanie kodu bledu
//
// Revision 1.10  2004/06/23 15:40:24  ltokarski
// Z0005 - sprawdzanie wybranego protokolu
//
// Revision 1.9  2004/06/23 12:46:36  ltokarski
// Z0004 - zmiana nazw UPsow na protokoly przy szukaniu UPSow
//
// Revision 1.7  2004/06/09 11:32:03  ltokarski
// nagwki CVS
// standard plikw Unix
//
///////////////////////////////////////////////////////

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include <signal.h>
#include <termios.h>
#include <string.h>
#include <ctype.h>
#include "linux_ty.h"
#include "comport.h"
#include "upsmon.h"
#include "ups.h"
#include "thread.h"
#include "upssmart.h"
#include "upssatur.h"
#include "upsjoint.h"
#include "upsorion.h"
#include "smem.h"

extern int __debug;

//////////////////////////
// zwraca ilosc sekund od poczatku pracy programu
time_t m_clock()
{
	time_t tm;
	static time_t tm_s;
	static time_t t_start=0;

	time(&tm);
	if(t_start==0)
	{
		t_start=1;
		time(&tm_s);
		return 0;
	}
	else
		return tm-tm_s;
}


/////////////////////////////////////////////////////////////////////////////
CUPS::CUPS(CUPSThreadInfo *Info)
{
	m_Info=Info;
} // CUPS::CUPS


/////////////////////////////////////////////////////////////////////////////
CUPS::~CUPS()
{
} // CUPS::~CUPS



/////////////////////////////////////////////////////////////////////////////
void CUPS::Start()
{
	// powiadamiamy glowny watek o rozpoczeciu pracy

	m_Info->m_CS.Lock();
	m_Info->m_IsRunning=1;
	m_Info->m_CS.Unlock();
	// czekamy na potwierdzenie
	int isRunning=1;
	while(isRunning==1)
	{
			//printf("Ups start3a %d \n",m_Info->m_IsRunning);
		m_Info->m_CS.Lock();
		isRunning=m_Info->m_IsRunning;
		m_Info->m_CS.Unlock();
	}
} // CUPS::Start


/////////////////////////////////////////////////////////////////////////////
void CUPS::Run(UINT protocol)	//Z0005, LT - wybor protokolu
{
	// poszczegolne rodzaje UPS'ow sa w osobnych blokach aby zawsze utworzony
	// byl jeden UPS (niewielka oszczednosc pamieci)
	if(protocol & PROTOCOL_SEC26)	//Z0005, LT
	{
//		printf("  Try Smartmoon protocol\n");
		printf("  Sprawdzanie UPSa z protokolem SEC 2.6...\n");
		CUPSSmartmon ups(m_Info);
		if(ups.StartMonitoring())
			return;
		m_Info->m_UPSStatus=-1;
	}
	if(protocol & PROTOCOL_MEGATEC)	//Z0005, LT
	{
//		printf("  Try Saturn protocol\n");
		printf("  Sprawdzanie UPSa z protokolem Megatec...\n");
		CUPSSaturn ups(m_Info);
		if(ups.StartMonitoring())
			return;
		m_Info->m_UPSStatus=-1;
	}
	// UPS z interfejsem stykowym powinien byc sprawdzany jako ostatni, poniewaz
	// pozostale UPS'y tez moga zmieniac sygnaly sterujace i moglyby omylkowo
	// zostac uznane za UPS z interfejsem stykowym
	if(protocol & PROTOCOL_NEWAVE)	//Z0005, LT
	{
//		printf("  Try Orion protocol\n");
		printf("  Sprawdzanie UPSa z protokolem Newave...\n");
		CUPSOrion ups(m_Info);
		if(ups.StartMonitoring())
		  return;
		m_Info->m_UPSStatus=-1;
	}
	if(protocol & PROTOCOL_JOINT)	//Z0005, LT
	{
//		printf("  Try Joint protocol\n");
		printf("  Sprawdzanie UPSa stykowego...\n");
		CUPSJoint ups(m_Info);
		//m_Info->m_UPSStatus=UPS_JOINT_ACTIVE;
		//printf("JJJ UpsS=%d\n",m_Info->m_UPSStatus);
		if(ups.StartMonitoring())
			return;
		m_Info->m_UPSStatus=-1;
	}

	//::PostMessage(m_Info->m_MainWnd,WM_USER_UPS_NOTIFICATION,STATUS_NO_COMMUNICATION,0);
//	printf("- Not found Ups.\n");
	printf("+ Nie wykryto UPSa!\n");
} // CUPS::Run


/////////////////////////////////////////////////////////////////////////////
void CUPS::End()
{
	// informujemy glowny watek ze zakonczylismy prace
	m_Info->m_CS.Lock();
	m_Info->m_IsRunning=0;
	m_Info->m_CS.Unlock();
} // CUPS::End



/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
CUPSGeneric::CUPSGeneric(CUPSThreadInfo *Info)
{
	m_Info=Info;
	m_ComPort.m_Info = Info;	//Z0008, LT - do zwrocenia kodu zakonczenia procesu
} // CUPSGeneric::CUPSGeneric


/////////////////////////////////////////////////////////////////////////////
CUPSGeneric::~CUPSGeneric()
{
} // CUPSGeneric::~CUPSGeneric


/////////////////////////////////////////////////////////////////////////////
// zwraca FALSE gdy nie znaleziono UPS'a, w przeciwnym razie (gdy znaleziono
// UPS'a i nalezy zakonczyc monitorowanie) zwraca TRUE
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSGeneric::StartMonitoring()
{
	m_Info->m_CS.Lock();
	BYTE action=m_Info->m_ActionType;
	m_Info->m_CommunicationOK=FALSE;
	m_Info->m_CanAutostart=0;
	m_Info->m_CanTurnOff=0;
	m_Info->m_AvpMan=0;
	m_Info->m_AvpMod=0;
	m_Info->m_AvpSoftVer=0;
	memset(m_Info->m_Manufacturer,0xFF,LEN_MAN);
	memset(m_Info->m_Model,0xFF,LEN_MODEL);
	memset(m_Info->m_SoftVer,0xFF,LEN_SOFT_VER);
	m_Info->m_BatCondition=-1;
	m_Info->m_BatStatus=-1;
	m_Info->m_BatVol=-1;
	m_Info->m_InVol=-1;
	m_Info->m_OutCurrent=-1;
	m_Info->m_OutLoad=-1;
	m_Info->m_OutPower=-1;
	m_Info->m_OutSource=-1;
	m_Info->m_OutVol=-1;
	//int port=m_Info->m_RSPort;
	m_Info->m_CS.Unlock();

	m_CommunicationOK=FALSE;
	m_Status=STATUS_GETTING_DATA;
	m_BatStatus=-1;
	m_OutSource=-1;

	if(!OpenUPS())
	{
		//printf("Error OpenUps\n");
		return FALSE;
	}

	if(!GetStaticData() || !GetDynamicData())
	{
		if(!CheckAction(action))
		{

			// jezeli action==ACTION_NO_BREAK to nie znaleziono UPSa
			if(__debug)
				printf("+ Close UPS\n");
			CloseUPS();
			return FALSE;
		}

		// action!=ACTION_NO_BREAK (na tym etapie moze byc tylko jeszcze
		// action==ACTION_TERMINATE), czyli nalezy zakonczyc monitorowanie
		if(__debug)
			printf("Close UPS 2\n");
		CloseUPS();
		return TRUE;
	}

	// ustawiamy zmienna informujaca ze zmienne o stanie UPS sa dostepne
	m_CommunicationOK=TRUE;
	m_Info->m_CS.Lock();
	m_Info->m_CommunicationOK=m_CommunicationOK;
	m_Info->shmem->Set(0, 1, SHARED_MEM_RESULT);	//Z0008, LT - poinformowanie procesu macierz o sukcesie
	m_Info->m_Protocol = m_Protocol;	//Z0013, LT
	m_Info->m_CS.Unlock();
	UpdateUPSStatus();

	clock_t start=m_clock();
	long sleepTime;
	while(action!=ACTION_TERMINATE && !ThreadCancel)
	{
		if(GetDynamicData())
		{
			if(!m_CommunicationOK)
			{
				m_CommunicationOK=TRUE;
				m_Info->m_CS.Lock();
				m_Info->m_CommunicationOK=m_CommunicationOK;
				m_Info->m_Protocol = m_Protocol;	//Z0013, LT
				m_Info->m_CS.Unlock();
			}
		}
		else
		{
			if(!CheckAction(action) && m_CommunicationOK)
			{
//printf("Action %d =Terminate %d\n",action,ACTION_TERMINATE);
				m_CommunicationOK=FALSE;
				m_Info->m_CS.Lock();
				m_Info->m_CommunicationOK=m_CommunicationOK;
				m_Info->m_Protocol = 0;		//Z0013, LT
				m_Info->m_CS.Unlock();
			}
		}

		CheckAction(action);
//printf("Action2 %d =Terminate %d\n",action,ACTION_TERMINATE);
		if(action==ACTION_TERMINATE)
			break;
		UpdateUPSStatus();
		if(action!=ACTION_NO_BREAK)
			DoAction(action);

		// wartosc uspienia jest dobierana tak aby co sekunde odswiezac status UPS'a
		sleepTime=(long)(CLOCKS_PER_SEC-(m_clock()-start));
//printf("Sleep %d\n",(sleepTime*1000)/CLOCKS_PER_SEC);
		sleepTime=(long)(CLOCKS_PER_SEC-(m_clock()-start));
		if(sleepTime>0)
		  	//usleep(1000*1000*(sleepTime*1000)/CLOCKS_PER_SEC);
			usleep(900000);
		start=m_clock();
	};
	if(__debug)
		printf("Close UPS Joint\n");
	CloseUPS();
	return TRUE;   // nalezy zakonczyc monitorowanie, wiec zwracamy TRUE
} // CUPSGeneric::StartMonitoring


/////////////////////////////////////////////////////////////////////////////
void CUPSGeneric::UpdateUPSStatus()
{
	BYTE status;
	if(m_OutSource==-1 || m_BatStatus==-1 || !m_CommunicationOK)
		status=STATUS_NO_COMMUNICATION;
	else if(m_OutSource==0)
		status=STATUS_NORMAL;
	else if(m_OutSource==2)
		status=STATUS_BYPASS;
	else if(m_OutSource==5)
		status=STATUS_OFF;
	else if(m_OutSource==3)
		status=STATUS_AVR_UP;
	else if(m_OutSource==4)
		status=STATUS_AVR_DOWN;
	else if(m_OutSource==1)
	{
		if(m_BatStatus>0)
			status=STATUS_BAT_LOW;
		else
			status=STATUS_BAT;
	}
	if(status!=m_Status)
	{
		m_Status=status;
		m_Info->m_Status = status;	//Z0012, LT - potrzebne do wyswietlenia
		//::PostMessage(m_Info->m_MainWnd,WM_USER_UPS_NOTIFICATION,m_Status,0);
	}
} // CUPSGeneric::GetUPSStatus


/////////////////////////////////////////////////////////////////////////////
BYTE CUPSGeneric::CheckAction(BYTE &ActionType)
{
	m_Info->m_CS.Lock();
	ActionType=m_Info->m_ActionType;
	m_Info->m_CS.Unlock();
	return ActionType;
} // CUPSGeneric::CheckAction


/////////////////////////////////////////////////////////////////////////////
void CUPSGeneric::DoAction(BYTE ActionType)
{
	//m_ComPort.FlushAll();

	m_Info->m_CS.Lock();
	m_Info->m_ActionType=ACTION_NO_BREAK;
	m_Info->m_CS.Unlock();

	if(__debug)
		printf("UPS DoAction %d\n",ActionType);
	BOOL retVal=FALSE;
	if(ActionType==ACTION_SET_AUTOSTART)
		retVal=DoSetAutostart(TRUE); //2
	else if(ActionType==ACTION_UNSET_AUTOSTART)
		retVal=DoSetAutostart(FALSE);//3
	else if(ActionType==ACTION_TURN_OFF_AUTOSTART)
		retVal=DoTurnOff(TRUE);//4
	else if(ActionType==ACTION_TURN_OFF)
		retVal=DoTurnOff(FALSE);            //5
	else if(ActionType==ACTION_JOINT_TURN_OFF_9X)
		retVal=DoTurnOff(FALSE,1);//6
	else if(ActionType==ACTION_JOINT_TURN_OFF_NT)
		retVal=DoTurnOff(FALSE,2);//7
	else if(ActionType==ACTION_RESTART)
		retVal=DoRestart();//8
	//	2003.01.13, MW, dodana obsluga trybu wylaczania UPS stykowego bez zamykania OS
	else if(ActionType==ACTION_TURN_OFF_JUPS_ONLY)    //9
		retVal = DoTurnOff(FALSE, 3);
	//	2003.01.13, MW, koniec
	//  2003.01.15, MW, poprawa obslugi wylacznia UPS-a stykowego bez zamykania systemu
	else if(ActionType==ACTION_END_TURN_OFF_JUPS)//10
		retVal = DoTurnOff(FALSE, 4);
	//  2003.01.15, MW, koniec
	// informujemy o zakonczeniu i zwracamy rezultat

	m_Info->m_CS.Lock();
	m_Info->m_ActionInProgress=FALSE;
	m_Info->m_RetVal=retVal;
//	m_Info->m_ActionInProgress=FALSE;
	m_Info->m_CS.Unlock();
//	if(__debug)
//		printf("CUPSGeneric::DoAction return %d\n", retVal);
} // CUPSGeneric::DoAction

// EOF //////////////////////////////////////////////////////////////////////
