///////////////////////////////////////////////////////
// Fideltronik R&D Department, Copyright (c)2003
//         $Author: ltokarski $
//         $Date: 2004/07/20 10:18:55 $
//         $Locker:  $
//         $Revision: 1.13 $
///////////////////////////////////////////////////////
//	Opis funkcjonalny moduu:
//          Komunikacja z zasilaczami stykowymi
///////////////////////////////////////////////////////
// Historia zmian:
// $Log: upsjoint.cpp,v $
// Revision 1.13  2004/07/20 10:18:55  ltokarski
// Z0013 - wypisywanie w statusie protokolu komunikacji
//
// Revision 1.12  2004/07/06 12:11:55  ltokarski
// Z0011 - ignorowanie CLOSE_SYSTEM_RUNUPS w stykowcach
//
// Revision 1.11  2004/06/25 11:42:18  ltokarski
// Z0007 - przywrocenie zapisu do upsoff.conf
//
// Revision 1.10  2004/06/09 11:32:03  ltokarski
// nagwki CVS
// standard plikw Unix
//
///////////////////////////////////////////////////////

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <signal.h>
#include <termios.h>
#include <string.h>
#include <ctype.h>

#include "linux_ty.h"
#include "comport.h"
#include "upsmon.h"
#include "ups.h"
#include "upsjoint.h"
#include "thread.h"


extern int __debug;

//#define  debug_rs

/////////////////////////////////////////////////////////////////////////////
//#define COM_CONTROL_STRING   "baud=1200 parity=N data=8 stop=1"
#define COM_CONTROL_STRING   "1200,N,8,1"


time_t CUPSJoint::m_clock()
{
	time_t tm;
	static time_t tm_s;
	static time_t t_start=0;

	time(&tm);
	if(t_start==0)
	{
		t_start=1;
		time(&tm_s);
		return 0;
	}
	else
		return tm-tm_s;
}

/////////////////////////////////////////////////////////////////////////////
CUPSJoint::CUPSJoint(CUPSThreadInfo *Info) : CUPSGeneric(Info)
{
//	2003.01.10, MW, UPS stykowy sygnalizuje swoj obecno na
	//	sowie stanu UPS-a, +eby klient mia informacj jak obsugiwa terminarz
	m_Info->m_CS.Lock();
	m_Info->m_UPSStatus = UPS_JOINT_ACTIVE;
	m_Info->m_CS.Unlock();
	m_IsTurningOff = FALSE;		// 2003.01.16, MW, poprawa obslugi wylaczania UPS-a stykowego bez zamykania OS
	m_Protocol = 1;		//Z0013, LT
} // CUPSJoint::CUPSJoint


/////////////////////////////////////////////////////////////////////////////
CUPSJoint::~CUPSJoint()
{
} // CUPSJoint::~CUPSJoint


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSJoint::OpenUPS()
{
#ifdef debug_rs
	return TRUE;
#endif

//	m_Info->m_CS.Lock();
//	int port=m_Info->m_RSPort;
//	m_Info->m_CS.Unlock();
//	if(!m_ComPort.Open(port,COM_CONTROL_STRING))
	if(!m_ComPort.Open(m_Info->m_RSPort, COM_CONTROL_STRING))
		  return FALSE;
	if(!m_ComPort.IsOpen()) return FALSE;
	return TRUE;
} // CUPSJoint::OpenUPS


/////////////////////////////////////////////////////////////////////////////
void CUPSJoint::CloseUPS()
{
#ifdef debug_rs
	return;
#endif
	// ustawiamy takie sygnaly jakie maja zostac, zeby UPS nie wylaczyl sie
	if(__debug)
		printf("Close UPS Set\n");
	m_ComPort.SetState(0,1);
	m_ComPort.Close();
} // CUPSJoint::CloseUPS


/////////////////////////////////////////////////////////////////////////////
// pobiera dane statyczne (w ten sposob sprawdza tez czy UPS jest obecny)
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSJoint::GetStaticData()
{
	BYTE cts, dcd;
	BOOL noUPS=FALSE;
	static int display=0;
#ifndef debug_rs
	m_Info->m_UPSStatus|=UPS_JOINT_ACTIVE;
	if(!noUPS && !m_ComPort.SetState(1,1))
		noUPS=TRUE;
    usleep(100000);
	if(!noUPS && !m_ComPort.GetState(cts,dcd))
		noUPS=TRUE;
	// sprawdzenie czy UPS jest podlaczony do portu, jesli
	// tak to na DCD powinno byc to co na DTR (czyli 1)
	if(!noUPS && dcd==0)
		noUPS=TRUE;
	if(!noUPS && !m_ComPort.SetState(1,0))
		noUPS=TRUE;
    usleep(100000);
	if(!noUPS && !m_ComPort.GetState(cts,dcd))
		noUPS=TRUE;
	// sprawdzenie czy UPS jest podlaczony do portu, jesli
	// tak to na DCD powinno byc to co na DTR (czyli 0)
#endif
#ifdef debug_rs
	if(__debug)
		printf("-------DEBUG !!!!1\n");
	noUPS=0;
	dcd=0;
#endif

	if(!noUPS && dcd==0)
	{
		// ustawiamy takie sygnaly jakie maja zostac
#ifndef debug_rs
//	2003.01.16, MW, poprawa obsugi wyczania UPS-a stykowego bez zamykania systemu
		//// ustawiamy takie sygnaly jakie maja zostac
		//if(!m_ComPort.SetState(0,1)) return FALSE;
		if(m_IsTurningOff==FALSE)	// 2003.01.16, MW, poprawa obslugi wylaczania UPS-a stykowego bez zamykania OS
			if(!m_ComPort.SetState(0,1))
				return FALSE;
		//	2003.01.16, MW, koniec

		m_CTS=cts;
		m_DCD=dcd;
#endif
#ifdef debug_rs
		m_CTS=1;
		m_DCD=1;
#endif

		// UPS z interfejsem stykowym, mozna go tylko wylaczyc,
		// brak jest danych statycznych, a dostepne sa tylko
		// zmienne: Battery Status i Output Source
		m_Info->m_CS.Lock();
		m_Info->m_CanAutostart=0;
		m_Info->m_CanTurnOff=1;
		m_Info->m_AvpMan=0;
		m_Info->m_AvpMod=0;
		m_Info->m_AvpSoftVer=0;
		memset(m_Info->m_Manufacturer,0xFF,LEN_MAN);
		memset(m_Info->m_Model,0xFF,LEN_MODEL);
		memset(m_Info->m_SoftVer,0xFF,LEN_SOFT_VER);
		m_Info->m_BatAutonomy=-1;
		m_Info->m_CS.Unlock();
		if(__debug)
			printf("+ Ups joint SUCCESS\n");
		if(display==0)
		{
			display=1;
			printf("+ Wykryty UPS stykowy\n");
		}
		return TRUE;
	}
	if(noUPS)
	{
		if(__debug)
			printf("Nie ups Ponowne SetState\n");
		// ustawiamy takie sygnaly jakie maja zostac, zeby po ewentualnym
		// podlaczeniu UPSa nie wylaczyl sie on
#ifndef debug_rs
		m_ComPort.SetState(0,1);
#endif
	}
//getchar();
	return FALSE;
} // CUPSJoint::GetStaticData


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSJoint::GetDynamicData()
{
	BYTE cts, dcd;

	if(!m_CommunicationOK)
	{
		//printf("Brak komunikacji\n");
		if(!GetStaticData())
			return FALSE;
	}
//	2003.01.15, MW, poprawa obslugi wylaczania UPS-a stykowego
	if(m_IsTurningOff)	// 2003.01.16, MW, poprawa obslugi wylaczania UPS-a stykowego bez zamykania OS
	{
		m_Info->m_CS.Lock();
		m_Info->m_OutSource = m_OutSource = 5;		// UPS wylaczony
		m_Info->m_CS.Unlock();
		return TRUE;
	}
	 //	2003.01.15, MW, koniec


	// dostepne sa tylko: Battery Status i Output Source
	// CTS - 1 (siec OK), 0 (awaria zasilania)
	// DCD - 1 (baterie OK), 0 (wyczerpane baterie)
	// gdy nie ma komunikacji to probujemy wykryc UPS
    usleep(100000);
	if(!m_ComPort.GetState(cts,dcd))
		return FALSE;

	// jezeli bylo CTS==1 i DCD==1 a jest CTS==0 i DCD==0
	// to traktujemy to jak wyrwanie kabla i zglaszamy blad komunikacji
#ifndef debug_rs
	if(m_CTS==1 && m_DCD==1 && cts==0 && dcd==0)
		return FALSE;
#endif

	if(cts==1)
		m_OutSource=0; // Normal
	else
		m_OutSource=1;       // On Battery
	if(dcd==1)
		m_BatStatus=0; // Battery OK
	else
		m_BatStatus=1;       // Battery Low
	// zapamietujemy stan sygnalow
	m_CTS=cts;
	m_DCD=dcd;

#ifdef debug_rs
	static int d=0;
	d++;
	if(d<60)
		m_OutSource=0; // Normal
	else
	{
		if(d>=60 && d<80)
		{
			m_OutSource=1;       // On Battery
			m_BatStatus=0; // Battery OK
		}
		else
		{
			if(d>=80 && d<100)
			{
				m_OutSource=1;       // On Battery
				m_BatStatus=1; 		// Battery LOW
			}
			else
				d=0;
		}
	}
#endif

	m_Info->m_CS.Lock();
	m_Info->m_OutSource=m_OutSource;
	m_Info->m_BatStatus=m_BatStatus;
	m_Info->m_CS.Unlock();
	return TRUE;
} // CUPSJoint::GetDynamicData


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSJoint::DoSetAutostart(BOOL Autostart)
{
	// ten protokol nie obsluguje ustawiania opcji autostartu, jednak
	// poniewaz w czasie zamykania systemu od lokalnego klienta przychodzi
	// pakiet UDP_PACKET_UPS_OFF i moze on miec flagi Autostart=0 i TurnOff=0
	// (Autostart jest 0 bo w kliencie ta opcja jest zabroniona dla tego UPSa,
	// a TurnOff jest 0 bo uzytkownik akurat nie chce wylaczyc UPS'a), to w
	// takiej sytuacji wywolywane jest DoSetAutostart(FALSE) i powinno sie to
	// zakonczyc sukcesem
	//Z0011 - nie ma potrzeby zwracac bledu przy ustawionym autostarcie
//	if(__debug)
//		printf("Joint AutoS=%d 0 blad  \n",Autostart);
//	if(!Autostart)
//		return TRUE;
//	return FALSE;
	return TRUE;
	//Z0011, LT - koniec
} // CUPSJoint::DoSetAutostart


/////////////////////////////////////////////////////////////////////////////
// JointCloseOpt == 0 - ustawienie flagi wylaczenia UPS
// JointCloseOpt == 1 - wylaczenie UPS dla wersji 9x
// JointCloseOpt == 2 - wylaczenie UPS dla wersji NT
// JointCloseOpt == 3 - wylaczenie UPS bez zamykania OS dla wersji 9x/NT
BOOL CUPSJoint::DoTurnOff(BOOL SetAutostart,BYTE JointCloseOpt)
{
	// ustawiamy opcje autostartu (patrz komentarz w funkcji DoSetAutostart())
	if(!DoSetAutostart(SetAutostart))
	{
		if(__debug)
			printf("Autostart FALSE\n");
		return FALSE;
	}
	// wylaczamy UPS, ustawiamy RTS=1 i DTR=0

	if(__debug)
		printf("JOINT Wylaczenie UPS %d \n",JointCloseOpt);
	if(JointCloseOpt==0)
	{
	//Z0007, LT - przywrocenie zapisu do upsoff.conf
		//LT: zapisuj plik przed potwierdzeniem klientowi
		//tu umozliwiam wylaczenie przez zewnetrzny program upsoff
		FILE *fdShutOff = fopen(m_Info->GetCfgPath(), "w");
		if(fdShutOff)
		{
//			fprintf(fdShutOff,"%d\n",1);
//			fprintf(fdShutOff,"/dev/ttyS%d\n",m_Info->m_RSPort);
			fprintf(fdShutOff, "1 %s\n", m_Info->m_RSPort);
			fclose(fdShutOff);
		}
		else
//			printf("Blad zapisu pliku\n");
			perror(m_Info->GetCfgPath());
		//LT koniec
	//Z0007, LT - koniec
		m_Info->m_CS.Lock();
		m_Info->m_JointTurnOffFlag=TRUE;
		m_Info->m_CS.Unlock();
		if(__debug)
			printf("m_Info->TurnOffFlag %d \n",m_Info->m_JointTurnOffFlag);
	}
	else
	{
		if(JointCloseOpt<3)
		{
			if(__debug)
				printf("Zapis pliku OFF %d\n",1);
			//LT: przeniesione wczeniej, przed potwierdzenie
//			//tu wylacznie przez zewnetrzny program
//			FILE *fdShutOff;
//			// tu nie znany numeru portu
//			fdShutOff = fopen("/etc/ups/scripts/upsoff.conf","w");
//			if (fdShutOff != NULL)
//			{
//				fprintf(fdShutOff,"%d\n",1);
//				fprintf(fdShutOff,"/dev/ttyS%d\n",m_Info->m_RSPort);
//				fclose(fdShutOff);
//			}
//			else
//				printf("Blad zapisu pliku\n");
		}
		else // (JointCloseOpt==3)
		{
			if(JointCloseOpt==3)
			{
				m_IsTurningOff = TRUE;	// 2003.01.16, MW, poprawa obslugi wylaczania UPS-a stykowego bez zamykania OS
				// wylaczamy UPS - ustawiamy RTS=1 i DTR=0
				m_ComPort.SetState(1,0);
				if(__debug)
					printf("Ustawiam 1,0\n");
				//	2003.01.15, MW, poprawa obslugi wylaczania UPS-a stykowego bez zamykania systemu
				//Sleep(10000);	// czekamy 10 sek. na wylaczenie UPS-a
				//m_ComPort.SetState(0,1); // zmiana sygnalow na takie, ktore nie wylaczaja UPS-a
				//	2003.01.15, MW, koniec
			}
			// 2003.01.15, MW, poprawa obslugi wylaczania UPS-a bez zamykania systemu
			else //(JointCloseOpt==4)
			{
				m_ComPort.SetState(0,1);	// zmiana sygnalow na takie, ktore nie wylaczaja UPS-a
				m_IsTurningOff = FALSE;		// 2003.01.16, MW, poprawa obslugi wylaczania UPS-a stykowego bez zamykania OS
			}
			// 2003.01.15, MW, koniec
		//	2003.01.13, MW, koniec
		}
	}//==0
	return TRUE;
} // CUPSJoint::DoTurnOff


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSJoint::DoRestart()
{
	// opcja restartu nie jest dostepna dla interfejsu stykowego
	return FALSE;
} // CUPSJoint::DoRestart

// EOF //////////////////////////////////////////////////////////////////////
