///////////////////////////////////////////////////////
// Fideltronik R&D Department, Copyright (c)2003
//         $Author: ltokarski $
//         $Date: 2004/07/20 10:18:55 $
//         $Locker:  $
//         $Revision: 1.6 $
///////////////////////////////////////////////////////
//	Opis funkcjonalny moduu:
//          Komunikacja z zasilaczami Orion
///////////////////////////////////////////////////////
// Historia zmian:
// $Log: upsorion.cpp,v $
// Revision 1.6  2004/07/20 10:18:55  ltokarski
// Z0013 - wypisywanie w statusie protokolu komunikacji
//
// Revision 1.5  2004/06/24 12:20:08  ltokarski
// Z0006 - konfigurowany czas opoznienia do wylaczenia (oprocz stykowych)
//
// Revision 1.4  2004/06/09 11:32:03  ltokarski
// nagwki CVS
// standard plikw Unix
//
///////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// UPSOrion.cpp
//	Zmiany:
//  2003.01.02 (MW)
//		- BOOL CUPSOrion::GetDynamicData() - odczytuje dodatkowo status
//		  z informacj o uszkodzeniach sprztowych UPS-a oraz warto
//		  autonomii akumulatorw (w %);

/////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include <signal.h>
#include <termios.h>
#include <string.h>
#include <ctype.h>

#include "linux_ty.h"
#include "comport.h"
#include "upsmon.h"
#include "ups.h"
#include "upsorion.h"
#include "thread.h"


/////////////////////////////////////////////////////////////////////////////
#define COM_CONTROL_STRING   "9600,N,8,1"
#define MAX_UPS_PACKET_LEN   80  // maksymalna dlugosc pakietow z UPSa
//#define RESPONSE_TIMEOUT     (CLOCKS_PER_SEC*1) // czas oczekiwania na pakiet z UPS'a
#define RESPONSE_TIMEOUT     1	// czas oczekiwania na pakiet z UPS'a
#define RESPONSE_RETRY       2    // ilosc retransmisji pakietu przed wyslaniem UPS_NO_COMMUNICATION
#define FIELD_MOD_LEN        20   // dlugosc pola UPS_Model w rozkazie S
#define FIELD_VER_LEN        20   // dlugosc pola Version w rozkazie S

static struct
{
	char	*Name;
	int		OutPower[6];
}	UPSModelData[] =
{
	{"Prime wave", {60, 45, 75,	100, 0, 0} },
	{"Power wave", {75, 100, 150, 200, 0, 0} },
	{"Eco wave",   {10, 15, 20, 30, 40, 60} },
	{"AEG",		   {60, 45, 75,	100, 0, 0} },
	{"t.b.d.",	   {60, 45, 75,	100, 0, 0} },
	{"Eco wave Parallel", {10, 15, 20, 30, 40, 60} }
};

//#define MAX_UPS_MODELS       (sizeof(UPSModels)/sizeof(UPSModels[0]))


/////////////////////////////////////////////////////////////////////////////
CUPSOrion::CUPSOrion(CUPSThreadInfo *Info) : CUPSGeneric(Info)
{
	m_Protocol = 4;		//Z0013, LT
} // CUPSOrion::CUPSOrion


/////////////////////////////////////////////////////////////////////////////
CUPSOrion::~CUPSOrion()
{
} // CUPSOrion::~CUPSOrion


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::OpenUPS()
{
//	int port;

//	m_Info->m_CS.Lock();
//	port = m_Info->m_RSPort;
//	m_Info->m_CS.Unlock();
//	m_ComPort.Open(port,COM_CONTROL_STRING);
	m_ComPort.Open(m_Info->m_RSPort, COM_CONTROL_STRING);
	if(!m_ComPort.IsOpen())
		return FALSE;
	// ustawiamy takie sygnaly zeby ewentualnie podlaczony UPS z interfejsem
	// stykowym nie wylaczyl sie
	m_ComPort.SetState(0,1);
	return TRUE;
}	// CUPSOrion::OpenUPS


/////////////////////////////////////////////////////////////////////////////
void CUPSOrion::CloseUPS()
{
	// ustawiamy takie sygnaly zeby ewentualnie podlaczony UPS z interfejsem
	// stykowym nie wylaczyl sie
	m_ComPort.SetState(0,1);
	m_ComPort.Close();
}	// CUPSOrion::CloseUPS


/////////////////////////////////////////////////////////////////////////////
// pobiera dane statyczne (w ten sposob sprawdza tez czy UPS jest obecny)
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::GetStaticData()
{
	BYTE	buff[MAX_UPS_PACKET_LEN+1], cmd[MAX_UPS_PACKET_LEN+1];
	char	mod[FIELD_MOD_LEN+1]={0};
	char	ver[FIELD_VER_LEN+1]={0};
	char	*ptr;
	int		buffLen=sizeof(buff), val;

	sprintf((char*)cmd, "\x1bS\r");
	if( !SendAndGetResponse((LPCTSTR)cmd, buff, buffLen) )
		return FALSE;
	if( (buffLen!=11)||((ptr = strchr((char*)buff, 'S'))==NULL) )
		return FALSE;
	ptr++;
	m_UPSType = *ptr - '4';				// konwersja indeksu z 4..9 ASCII na 0..5 binarnie;
	ptr += 6;
	if((m_UPSType!=2) && (m_UPSType!=5))
		m_UPSOutPower = *ptr&0x03;
	else
		m_UPSOutPower = *ptr&0x0f;

	char	Model[255];
	memset(Model,0,sizeof(Model));
	val = UPSModelData[m_UPSType].OutPower[m_UPSOutPower]/10;
	sprintf(Model,"%s %d.%d kVA", UPSModelData[m_UPSType].Name,
		val, UPSModelData[m_UPSType].OutPower[m_UPSOutPower]-(val*10));

//printf("Model-%s-\n",Model);
	memset(mod,0,sizeof(mod));
	memcpy(mod,Model,FIELD_MOD_LEN);
	for(int i=0; i<FIELD_MOD_LEN; i++)
		if(!isprint(mod[i]))
			mod[i]=' ';
	printf("+ Model: %s\n",mod);
	buffLen=sizeof(buff);
	sprintf((char*)cmd, "\x1bW\r");
	if(!SendAndGetResponse((char*)cmd, buff, buffLen))
		return FALSE;
	if((buffLen!=19)||((ptr = strchr((char*)buff, 'W'))==NULL))
		return FALSE;
	ptr++;
	*strchr((char*)buff, 0x0d) = 0x00;		// zakonczenie nazwy wersji oprogramowania zerem;
	memcpy(ver, ptr, FIELD_VER_LEN);
	for(int i=0; i<FIELD_VER_LEN; i++)
		if(!isprint(ver[i]))
			ver[i]=' ';
//	printf("+ Version %s\n",ver);
	printf("+ Wersja: %s\n",ver);
	m_Info->m_CS.Lock();
	m_Info->m_CanAutostart = 1;
	m_Info->m_CanTurnOff = 1;
	m_Info->m_AvpMan = 0;
	m_Info->m_AvpMod = 1;
	m_Info->m_AvpSoftVer = 1;
	memset(m_Info->m_Manufacturer, ' ', LEN_MAN);
	memset(m_Info->m_Model, ' ', LEN_MODEL);
	memcpy(m_Info->m_Model, mod, FIELD_MOD_LEN);
	memset(m_Info->m_SoftVer, ' ', LEN_SOFT_VER);
	memcpy(m_Info->m_SoftVer, ver, FIELD_VER_LEN);
	m_Info->m_CS.Unlock();
	return TRUE;
} // CUPSOrion::GetStaticData


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::GetDynamicData()
{
	BYTE	buff[MAX_UPS_PACKET_LEN+1], cmd[MAX_UPS_PACKET_LEN+1];
	int		value, buffLen=sizeof(buff);
	char	batCondition = -1;
	char	batStatus = -1;           // 22 - Battery Status
	short	batVol = -1;              // 24 - Battery Voltage
	short	inVol = -1;               // 51 - Input Voltage 1
	short	outCurrent = -1;          // 65 - Output Current 1
	short	outLoad = -1;             // 69 - Output Load 1
	long	outPower = -1;            // 73 - Output Power 1
	char	outSource = -1;           // 76 - Output Source
	short	outVol = -1;              // 77 - Output Voltage 1
	long	batAutonomy = -1;		  // (2003.01.02, MW)
	short	UPSStatus = -1;
	char*	ptr;

	sprintf((char*)cmd, "\x1bV00\r");
	if(!SendAndGetResponse((char*)cmd, buff, buffLen))
		return FALSE;
	if( (buffLen!=46)||((ptr=strchr((char*)buff, 'V'))==NULL) )
		return FALSE;
	ptr++;

	inVol = atoi(ptr)*10;				// input voltage value
	ptr += 4;
	outVol = atoi(ptr)*10;				// output voltage value
	ptr += 4;
	batVol = atoi(ptr)*10;				// positive battery
	ptr += 4;
	batVol += (atoi(ptr)*10);			// negative battery
	ptr += 4;
	outCurrent = atoi(ptr);				// calc output current
	ptr += 4;
	ptr += 4;							// don't care about phase control voltage
	outPower = UPSModelData[m_UPSType].OutPower[m_UPSOutPower]*100;
	outPower *= atoi(ptr);				// actual power output (%)
	outPower /= 100;
	ptr += 4;

	//	(2003.01.02, MW)
	//	Wg dokumentacji teoretycznie do odczytu autonomii suy rozkaz
	//	ESC K CR, ale on zwraca zawsze sta warto 010 zamiast wartoci
	//  z przedziau 006..240. Tak warto uzyskuje si rozkazem V08, wic
	//  zrobiem to w oparciu o parametr 08 odpowiedzi na ESC V 00 CR,
	//  ale to moe w jakich przypadkach nie dziaa !!!
	batAutonomy = (atoi(ptr)*100)/240;
										// reszta parametrw nieuzywana
	buffLen = sizeof(buff);
	sprintf((char*)cmd, "\x1bS\r");
	if(!SendAndGetResponse((char*)cmd, buff, buffLen))
		return FALSE;
	if((buffLen!=11)||((ptr = strchr((char*)buff, 'S'))==NULL))
		return FALSE;
	//	ptr += 5; (2003.01.02, MW)
	//	Dodanie odczytu statusu UPS-a (2003.01.02, MW)
	ptr += 4;
	UPSStatus = 0;
	value = (*ptr)&0x0f;
	if(value&0x08)
		UPSStatus |= UPS_BCHARGER_FAULT;
	ptr++;
	//	koniec zmian (2003.01.02, MW)
/*
										// reszta parametrw nieuzywana
	buffLen = sizeof(buff);
	sprintf((char*)cmd, "\x1bS\r");
	if(!SendAndGetResponse((char*)cmd, buff, buffLen))
		return FALSE;
	if((buffLen!=11)||((ptr = strchr((char*)buff, 'S'))==NULL))
		return FALSE;
	ptr += 5;
*/
	value = (*ptr)&0x0f;

	if(value&0x01)
	{
		outSource = 0;				// praca sieciowa
		if(value&0x08)
			outSource = 2;			// on bypass
	}
	else
	{
		if(value&0x02)
			outSource = 1;			// praca bateryjna
		else
			outSource = 5;			// wyjscie wylaczone
	}
	batStatus = 0;
	if(value&0x04)
		batStatus = 1;				// battery low

	ptr ++;
	value = (*ptr)&0x0f;
	batCondition = 0;
	if((m_UPSType!=2) && (m_UPSType!=5))
	{
		if(value&0x04)
			batCondition = 2;			// baterie do wymiany
	}
	else
	{
		if(value&0x02)
			batCondition = 2;			// baterie do wymiany
	}
	m_Info->m_CS.Lock();
	m_Info->m_BatCondition = batCondition;
	m_Info->m_BatStatus = batStatus;
	m_Info->m_BatVol = batVol;
	m_Info->m_InVol = inVol;
	m_Info->m_OutCurrent = outCurrent;
	m_Info->m_OutLoad = outLoad;
	m_Info->m_OutPower = outPower;
	m_Info->m_OutSource = outSource;
	m_Info->m_OutVol = outVol;
	m_Info->m_BatAutonomy = batAutonomy;	// (2003.01.02, MW)
	m_Info->m_UPSStatus = UPSStatus;		//	(2003.01.02, MW)
	m_Info->m_CS.Unlock();
	m_BatStatus = batStatus;
	m_OutSource = outSource;
	return TRUE;
} // CUPSOrion::GetDynamicData


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::DoSetAutostart(BOOL Autostart)
{
	// ten protokol nie obsluguje ustawiania opcji autostartu, jednak
	// poniewaz w czasie zamykania systemu od lokalnego klienta przychodzi
	// pakiet UDP_PACKET_UPS_OFF i moze on miec flagi Autostart=0 i TurnOff=0
	// (Autostart jest 0 bo w kliencie ta opcja jest zabroniona dla tego UPSa,
	// a TurnOff jest 0 bo uzytkownik akurat nie chce wylaczyc UPS'a), to w
	// takiej sytuacji wywolywane jest DoSetAutostart(FALSE) i powinno sie to
	// zakonczyc sukcesem
	if(!Autostart)
		return TRUE;
	return FALSE;
} // CUPSOrion::DoSetAutostart


/////////////////////////////////////////////////////////////////////////////
// JointCloseOpt - parametr wykorzystywany tylko dla UPS z interfejsem stykowym
BOOL CUPSOrion::DoTurnOff(BOOL SetAutostart, BYTE)
{
	BYTE	buff[MAX_UPS_PACKET_LEN+1];
	int		buffLen=sizeof(buff);

	// ustawiamy opcje autostartu (patrz komentarz w funkcji DoSetAutostart())
	if(!DoSetAutostart(SetAutostart))
		return FALSE;

	// sprawdzamy czy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) jest poprawna
//	if(UPS_SHUTDOWN_TIME>600)
	if(m_Info->GetTurnOffDelay() > 600)	//Z0006, LT
		return FALSE; // maksymalnie moze byc 10 min.

	// przeliczamy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) na minuty
	// i przygotowujemy pakiet S
	char cmd[MAX_UPS_PACKET_LEN+1];

//	if(UPS_SHUTDOWN_TIME>=60)
//		sprintf(cmd,"\x1bX%02d\r",UPS_SHUTDOWN_TIME/60);
	if(m_Info->GetTurnOffDelay() >= 60)	//Z0006, LT
		sprintf(cmd,"\x1bX%02d\r", m_Info->GetTurnOffDelay()/60);
	else
		sprintf(cmd,"\x1bX01\r"); // minimalnie moze byc 1 min.

	// wysylamy rozkaz
	if(!SendAndGetResponse(cmd, buff, buffLen))
		return FALSE;

	return TRUE;
} // CUPSOrion::DoTurnOff


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::DoRestart()
{
	BYTE	buff[MAX_UPS_PACKET_LEN+1];
	int		buffLen=sizeof(buff);

	m_Info->m_CS.Lock();
	long delay=m_Info->m_ActionData; // opoznienie (w sek.) ponownego zalaczenia UPS'a
	m_Info->m_CS.Unlock();

	// przeliczamy opoznienie zalaczenia UPS'a na min. i sprawdzamy czy
	// miesci sie w dopuszczalnym zakresie
	delay/=60;
	if(delay<1 || delay>9999)
		return FALSE;

	// sprawdzamy czy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) jest poprawna
//	if((UPS_SHUTDOWN_TIME==0) || (UPS_SHUTDOWN_TIME>599940L))
	if(m_Info->GetTurnOffDelay() == 0 || m_Info->GetTurnOffDelay() > 599940L)	//Z0006, LT
		return FALSE; // moze byc 0 < czas <=9999

	// przeliczamy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) na minuty
	// i przygotowujemy pakiet SR
	char cmd[MAX_UPS_PACKET_LEN+1];

	sprintf(cmd,"\x1bO%04d\r",(int)delay);
	// wysylamy rozkaz
	if(!SendAndGetResponse(cmd, buff, buffLen))
		return FALSE;

//	if(UPS_SHUTDOWN_TIME>=60)
//		sprintf(cmd,"\x1bP%02d\r",UPS_SHUTDOWN_TIME/60);
	if(m_Info->GetTurnOffDelay() >= 60)	//Z0006, LT
		sprintf(cmd,"\x1bP%02d\r", m_Info->GetTurnOffDelay()/60);
	else
		sprintf(cmd,"\x1bP01\r"); // minimalnie moze byc 1 min.
	// wysylamy rozkaz
	if(!SendAndGetResponse(cmd, buff, buffLen))
		return FALSE;
	return TRUE;
} // CUPSOrion::DoRestart


/////////////////////////////////////////////////////////////////////////////
// wysyla (z ewentualna retransmisja) pakiet Cmd i czeka na odpowiedz
// zwraca TRUE gdy odebrano odpowiedz
// zwraca FALSE gdy wystapil timeout, lub m_Info->m_ActionType!=ACTION_NO_BREAK
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::SendAndGetResponse(LPCTSTR Cmd, BYTE *Buff, int &Len)
{
	BYTE action;
	int l;
	int retry=0;

	while(++retry<=RESPONSE_RETRY)
	{
		SendPacket(Cmd);
		l = Len;
		if(GetPacket(Buff, l, Cmd))
		{
			if(*(Buff+1)==(BYTE)*(Cmd+1))		// czy odebrano rzeczywiscie odpowiedz na rozkaz
			{
				Len = l;
				return TRUE;
			}
		}
		if(CheckAction(action))
			return FALSE;
	}
	return FALSE;
} // CUPSOrion::SendAndGetResponse


/////////////////////////////////////////////////////////////////////////////
void CUPSOrion::SendPacket(LPCTSTR Cmd)
{
	m_ComPort.WriteStr((BYTE*)Cmd,strlen(Cmd));
} // CUPSOrion::SendPacket


/////////////////////////////////////////////////////////////////////////////
// zwraca TRUE gdy odebrano pakiet
// zwraca FALSE gdy wystapil timeout, lub m_Info->m_ActionType!=ACTION_NO_BREAK
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::GetPacket(BYTE *Buff, int &Len, LPCTSTR Cmd)
{
	int ch, pos=0;
	BYTE action;
//	clock_t start=clock();
//	while((clock()-start)<RESPONSE_TIMEOUT)
	time_t tm;
	time_t start=time(&tm);
	while((time(&tm)-start)<RESPONSE_TIMEOUT)
	{
		while((ch=m_ComPort.ReadChar())!=-1)
		{
			if(ch=='\x1b')
				pos=0;
			if(pos>=Len)
				continue;
			Buff[pos++]=ch;
			if(CheckPacket(Buff,pos,Cmd))
			{
				Buff[pos]='\0';
				Len=pos;
                return TRUE;
			}
		}
		if(CheckAction(action))
			return FALSE;
		usleep(100000);
	}
	return FALSE;
} // CUPSOrion::GetPacket


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSOrion::CheckPacket(BYTE *Buff, int Len, LPCTSTR Cmd)
{
	if(Len<2)
		return FALSE;
	if(Buff[Len-1]!='\r')			// brak prawidlowego zakonczenia pakietu
		return FALSE;
	if(*Buff!='\x1b')				// brak prawidlowego rozpoczecia pakietu
		return FALSE;
	if(*(Buff+1)=='|')				// timeout (5 sekund)
		return FALSE;
	return TRUE;
} // CUPSOrion::CheckPacket


/////////////////////////////////////////////////////////////////////////////
long CUPSOrion::GetNum(BYTE *Buff, int &Offset)
{
	char *ptr;
	long num=strtol((char*)Buff+Offset,&ptr,10);
	Offset=(ptr-(char*)Buff)+1;
	return num;
} // CUPSOrion::GetNum

// EOF //////////////////////////////////////////////////////////////////////

