///////////////////////////////////////////////////////
// Fideltronik R&D Department, Copyright (c)2003
//         $Author: ltokarski $
//         $Date: 2004/07/20 10:18:55 $
//         $Locker:  $
//         $Revision: 1.7 $
///////////////////////////////////////////////////////
//	Opis funkcjonalny moduu:
//          Komunikacja z zasilaczami Saturn
///////////////////////////////////////////////////////
// Historia zmian:
// $Log: upssatur.cpp,v $
// Revision 1.7  2004/07/20 10:18:55  ltokarski
// Z0013 - wypisywanie w statusie protokolu komunikacji
//
// Revision 1.6  2004/06/24 12:20:08  ltokarski
// Z0006 - konfigurowany czas opoznienia do wylaczenia (oprocz stykowych)
//
// Revision 1.5  2004/06/09 11:32:03  ltokarski
// nagwki CVS
// standard plikw Unix
//
///////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include <signal.h>
#include <termios.h>
#include <string.h>
#include <ctype.h>
#include "linux_ty.h"
#include "comport.h"
#include "upsmon.h"
#include "ups.h"
#include "upssatur.h"
#include "thread.h"


/////////////////////////////////////////////////////////////////////////////
#define COM_CONTROL_STRING   "2400,N,8,1"
#define MAX_UPS_PACKET_LEN   80  // maksymalna dlugosc pakietow z UPSa
#define RESPONSE_TIMEOUT     2 // czas oczekiwania na pakiet z UPS'a
#define RESPONSE_RETRY       2    // ilosc retransmisji pakietu przed wyslaniem UPS_NO_COMMUNICATION
#define FIELD_MAN_LEN        15   // dlugosc pola Company_Name w rozkazie I
#define FIELD_MOD_LEN        10   // dlugosc pola UPS_Model w rozkazie I
#define FIELD_VER_LEN        10   // dlugosc pola Version w rozkazie I


extern int __debug;
static struct
{
	char *CompanyName;             // wartosc pola Company_Name w rozkazie I
	char *UPSModel;                // wartosc pola UPS_Model w rozkazie I
	short MaxCurr;                 // maksymalne natezenie (w 0.1 A)
	long  MaxPower;                // maksymalna moc (w 1 W)
	short CellsInBat;              // ilosc cel w bateriach
} UPSModels[]=
{
	{ "AEC", "NS1K6_220", 70, 1600, 24 },
	{ "ALLIS_EERDC", "NS2K_220", 104, 2400, 36 },
	{ "ALLIS_EERDC", "NS3K_220", 139, 3200, 48 },
	{ "AEC", "SR2010-220", 43, 1000, 24 },
	{ "AEC", "SR2020-220", 87, 2000, 36 },
	{ "AEC", "SR2030-220", 130, 3000, 48 },
	{ "AEC", "S1007-220", 43, 1000, 24 }
};

#define MAX_UPS_MODELS       (sizeof(UPSModels)/sizeof(UPSModels[0]))


//////////////////////////
// zwraca ilosc sekund od poczatku pracy programu
time_t CUPSSaturn::m_clock()
{
	time_t tm;
	static time_t tm_s;
	static time_t t_start=0;

	time(&tm);
	if(t_start==0)
	{
		t_start=1;
		time(&tm_s);
		return 0;
	}
	else
		return tm-tm_s;
}

/////////////////////////////////////////////////////////////////////////////
CUPSSaturn::CUPSSaturn(CUPSThreadInfo *Info) : CUPSGeneric(Info)
{
	m_Protocol = 3;		//Z0013, LT
} // CUPSSaturn::CUPSSaturn


/////////////////////////////////////////////////////////////////////////////
CUPSSaturn::~CUPSSaturn()
{
} // CUPSSaturn::~CUPSSaturn


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::OpenUPS()
{
//	m_Info->m_CS.Lock();
//	int port=m_Info->m_RSPort;
//	m_Info->m_CS.Unlock();
//	//printf("Open port %d\n",port);
//	m_ComPort.Open(port,COM_CONTROL_STRING);
	m_ComPort.Open(m_Info->m_RSPort, COM_CONTROL_STRING);
	if(!m_ComPort.IsOpen())
		return FALSE;
	return TRUE;
} // CUPSSaturn::OpenUPS


/////////////////////////////////////////////////////////////////////////////
void CUPSSaturn::CloseUPS()
{
	// ustawiamy takie sygnaly zeby ewentualnie podlaczony UPS z interfejsem
	// stykowym nie wylaczyl sie
	m_ComPort.SetState(0,1);
	m_ComPort.Close();
} // CUPSSaturn::CloseUPS


/////////////////////////////////////////////////////////////////////////////
// pobiera dane statyczne (w ten sposob sprawdza tez czy UPS jest obecny)
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::GetStaticData()
{
	BYTE buff[MAX_UPS_PACKET_LEN+1];
	int buffLen=sizeof(buff);

	if(!SendAndGetResponse("I\r",buff,buffLen))
		return FALSE;
	if(buffLen!=39)
		return FALSE;

	char man[FIELD_MAN_LEN+1]={0};
	char mod[FIELD_MOD_LEN+1]={0};
	char ver[FIELD_VER_LEN+1]={0};

	memcpy(man,buff+1,FIELD_MAN_LEN);
	for(int i=0; i<FIELD_MAN_LEN; i++)
		if(!isprint(man[i]))
			man[i]=' ';
//	printf("+ Manufacture %s.",man);
	printf("+ Producent: %s",man);
	if(man[strlen(man)-1]!='\n')
		printf("\n");

	memcpy(mod,buff+2+FIELD_MAN_LEN,FIELD_MOD_LEN);
	for(int i=0; i<FIELD_MOD_LEN; i++)
		if(!isprint(mod[i]))
			mod[i]=' ';
	printf("+ Model: %s.\n", mod);
	memcpy(ver,buff+3+FIELD_MAN_LEN+FIELD_MOD_LEN,FIELD_VER_LEN);
	for(int i=0; i<FIELD_VER_LEN; i++)
		if(!isprint(ver[i]))
			ver[i]=' ';
//	 printf("+ Version %s.\n");
	printf("+ Wersja: %s\n", ver);

	// szukamy tego UPSa w tablicy UPSModels aby okreslic wartosci
	// m_MaxCurr i m_CellsInBat
	m_MaxCurr=-1;      // na razie ustawiamy jako niedostepne
	m_MaxPower=-1;
	m_CellsInBat=-1;   // na razie ustawiamy jako niedostepne
	char company[120],tmp[255];
	int j;
	strcpy(company,man);
	memset(tmp,0,sizeof(tmp));
	j=0;
	for(UINT i=0; i < strlen(company); i++)
		if(company[i]!=' ')
			tmp[j++]=company[i];
	strcpy(company,tmp);
	memset(tmp,0,sizeof(tmp));
	char model[120];
	strcpy(model,mod);
	j=0;
	for(UINT i=0; i < strlen(mod); i++)
		if(mod[i]!=' ')
			tmp[j++]=mod[i];
	strcpy(model,tmp);
	//company.TrimLeft();
	//company.TrimRight();
	//CString model=mod;
	//model.TrimLeft();
	//model.TrimRight();
	for(int i=0; i < (int)MAX_UPS_MODELS; i++)
	{
//   printf("--%s,%s-- i --%s,%s--\n",company,UPSModels[i].CompanyName,model,UPSModels[i].UPSModel);
		if(( strcmp(company,UPSModels[i].CompanyName)==0) && ( strcmp(model,UPSModels[i].UPSModel)==0))
		{
			m_MaxCurr=UPSModels[i].MaxCurr;
			m_CellsInBat=UPSModels[i].CellsInBat;
			break;
		}
	}

	m_Info->m_CS.Lock();
	m_Info->m_CanAutostart=0;
	m_Info->m_CanTurnOff=1;
	m_Info->m_AvpMan=1;
	m_Info->m_AvpMod=1;
	m_Info->m_AvpSoftVer=1;
	memset(m_Info->m_Manufacturer,' ',LEN_MAN);
	memcpy(m_Info->m_Manufacturer,man,FIELD_MAN_LEN);
	memset(m_Info->m_Model,' ',LEN_MODEL);
	memcpy(m_Info->m_Model,mod,FIELD_MOD_LEN);
	memset(m_Info->m_SoftVer,' ',LEN_SOFT_VER);
	memcpy(m_Info->m_SoftVer,ver,FIELD_VER_LEN);
	m_Info->m_BatAutonomy=-1;
	m_Info->m_CS.Unlock();

	return TRUE;
} // CUPSSaturn::GetStaticData


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::GetDynamicData()
{
	BYTE buff[MAX_UPS_PACKET_LEN+1];
	int offset, buffLen=sizeof(buff);
	char batStatus=-1;            // 22 - Battery Status
	short batVol=-1;              // 24 - Battery Voltage
	short inVol=-1;               // 51 - Input Voltage 1
	short outCurrent=-1;          // 65 - Output Current 1
	short outLoad=-1;             // 69 - Output Load 1
	long outPower=-1;             // 73 - Output Power 1
	char outSource=-1;            // 76 - Output Source
	short outVol=-1;              // 77 - Output Voltage 1

	if(!SendAndGetResponse("Q1\r",buff,buffLen))
		return FALSE;
	if(buffLen!=47)
		return FALSE;
	// do pobierania zmiennych wykorzystujemy funkcje do odczytu liczb
	// calkowitych (osobno pobieramy czesc calkowita i osobna ulamkowa),
	// poniewaz jest to szybsze (i dla malych wartosci dokladniejsze) niz
	// wykorzystanie funkcji do odczytu liczb ulamkowych
	//printf("buff=%s\n",buff);
	offset=1; // ustawiamy offset po '('
	// pobieramy I/P Voltage przeksztalcajac na jednostki 0.1 V
	if(offset<buffLen)
		inVol=10*(short)GetNum(buff,offset); // czesc calkowita
	if(offset<buffLen)
		inVol+=(short)GetNum(buff,offset);   // czesc dziesietna
	offset=13; // pomijamy I/P Fault Voltage
	// pobieramy O/P Voltage przeksztalcajac na jednostki 0.1 V
	if(offset<buffLen)
		outVol=10*(short)GetNum(buff,offset); // czesc calkowita
	if(offset<buffLen)
		outVol+=(short)GetNum(buff,offset);   // czesc dziesietna
	// pobieramy O/P Current, ale poniewaz jest to w % to traktujemy to jako obciazenie
	if(offset<buffLen)
		outLoad=(short)GetNum(buff,offset);
	offset=28; // pomijamy I/P frequency
	// pobieramy Battery Voltage przeksztalcajac na jednostki 0.1 V
	short batVol_frac=0;
	if(offset<buffLen)
		batVol=10*(short)GetNum(buff,offset);   // czesc calkowita
	if(offset<buffLen)
		batVol_frac=(short)GetNum(buff,offset); // czesc ulamkowa
	//printf("----Bat %d,%d,%d\n",batVol,batVol_frac,m_CellsInBat);
	// pomijamy Temperature i pobieramy bity z UPS Status
	// buff[38] - Utility Fail
	if(buff[38]=='1')
		outSource=1; // 1 = On Battery
	else
		outSource=0;              // 0 = Normal
	// buff[39] - Battery Low
	if(buff[39]=='1')
		batStatus=1; // 1 = Battery Low
	else
		batStatus=0;              // 0 = Battery OK
	// buff[40] - Bypass/Boost or Buck Active
	if(buff[40]=='1')
		outSource=2; // 2 = On Bypass
	// buff[44] - Shutdown Active
	if(buff[44]=='1')
		outSource=5; // 5 = Other
	// buff[42] - UPS Type: 1 is Standby (0 is On-line)
	if(buff[42]=='0')
	{
		// dla UPS On-line napiecie baterii jest postaci S.SS (napiecie dla
		// pojedynczej celi) nalezy je przeliczyc na calkowite napiecie baterii
		batVol=(short)(((double)batVol+((double)batVol_frac/10.0))*m_CellsInBat);
		if(batVol<0)
			batVol=-1;
	}
	else
	{
		// dla UPS Standby napiecie baterii jest postaci SS.S nalezy wiec
		// tylko dodac czes ulamkowa
		batVol+=batVol_frac;
	}
	// jezeli znamy maksymalny prad to obliczamy Output Current i Output Power
	if(m_MaxCurr>=0)
	{
		outCurrent=(m_MaxCurr*outLoad)/100;           // Output Current (w 0.1 A)
		outPower=((long)outCurrent*(long)outVol)/100; // Output Power (w 1 W)
	}
	if(m_MaxPower>=0)
	{
		outPower=(m_MaxPower*(long)outLoad)/100; // Output Power (w 1 W)
	}


	m_Info->m_CS.Lock();
	m_Info->m_BatStatus=batStatus;
	m_Info->m_BatVol=batVol;
	m_Info->m_InVol=inVol;
	m_Info->m_OutCurrent=outCurrent;
	m_Info->m_OutLoad=outLoad;
	m_Info->m_OutPower=outPower;
	m_Info->m_OutSource=outSource;
	m_Info->m_OutVol=outVol;
	m_Info->m_CS.Unlock();
	m_BatStatus=batStatus;
	m_OutSource=outSource;

	return TRUE;
} // CUPSSaturn::GetDynamicData


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::DoSetAutostart(BOOL Autostart)
{
	// ten protokol nie obsluguje ustawiania opcji autostartu, jednak
	// poniewaz w czasie zamykania systemu od lokalnego klienta przychodzi
	// pakiet UDP_PACKET_UPS_OFF i moze on miec flagi Autostart=0 i TurnOff=0
	// (Autostart jest 0 bo w kliencie ta opcja jest zabroniona dla tego UPSa,
	// a TurnOff jest 0 bo uzytkownik akurat nie chce wylaczyc UPS'a), to w
	// takiej sytuacji wywolywane jest DoSetAutostart(FALSE) i powinno sie to
	// zakonczyc sukcesem
	if(!Autostart)
		return TRUE;
	return FALSE;
} // CUPSSaturn::DoSetAutostart


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::DoTurnOff(BOOL SetAutostart, BYTE)
{
	// ustawiamy opcje autostartu (patrz komentarz w funkcji DoSetAutostart())
	if(!DoSetAutostart(SetAutostart))
		return FALSE;

	// sprawdzamy czy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) jest poprawna
//	if(UPS_SHUTDOWN_TIME>600)
	if(m_Info->GetTurnOffDelay() > 600)	//Z0006, LT
		return FALSE; // maksymalnie moze byc 10 min.

	// przeliczamy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) na minuty
	// i przygotowujemy pakiet S
	char cmd[MAX_UPS_PACKET_LEN+1];
//	if(UPS_SHUTDOWN_TIME>=60)
//		sprintf(cmd,"S%02d\r",UPS_SHUTDOWN_TIME/60);
	if(m_Info->GetTurnOffDelay() >= 60)	//Z0006, LT
		sprintf(cmd,"S%02d\r", m_Info->GetTurnOffDelay()/60);
	else
	{
	//Z0006, LT - teraz czas jest zmienna, brakowalo tez else!
//		if(UPS_SHUTDOWN_TIME<12)
//			sprintf(cmd,"S.2\r"); // minimalnie moze byc .2 min.
//		sprintf(cmd,"S.%d\r",UPS_SHUTDOWN_TIME/6);
		if(m_Info->GetTurnOffDelay() < 12)
			sprintf(cmd,"S.2\r"); // minimalnie moze byc .2 min.
		else
			sprintf(cmd,"S.%d\r", m_Info->GetTurnOffDelay()/6);
	//Z0006, LT - koniec
	}

	// wysylamy rozkaz (nie ma on potwierdzenia)
	SendPacket(cmd);
	usleep(1200000);
	SendPacket(cmd);
	usleep(1200000);
	SendPacket(cmd);
	return TRUE;
} // CUPSSaturn::DoTurnOff


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::DoRestart()
{
	m_Info->m_CS.Lock();
	long delay=m_Info->m_ActionData; // opoznienie (w sek.) ponownego zalaczenia UPS'a
	m_Info->m_CS.Unlock();

	// przeliczamy opoznienie zalaczenia UPS'a na min. i sprawdzamy czy
	// miesci sie w dopuszczalnym zakresie
	delay/=60;
	if(delay<1 || delay>9999)
		return FALSE;

	// sprawdzamy czy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) jest poprawna
//	if(UPS_SHUTDOWN_TIME>600)
	if(m_Info->GetTurnOffDelay() > 600)	//Z0006, LT
		return FALSE; // maksymalnie moze byc 10 min.

	// przeliczamy wartosc UPS_SHUTDOWN_TIME (ktora jest w sek.) na minuty
	// i przygotowujemy pakiet SR
	char cmd[MAX_UPS_PACKET_LEN+1];
	//Z0006, LT - teraz czas jest zmienna, brakowalo tez else!
//	if(UPS_SHUTDOWN_TIME>=60)
//		sprintf(cmd,"S%02dR%04ld\r",UPS_SHUTDOWN_TIME/60,delay);
//	else
//	{
//		if(UPS_SHUTDOWN_TIME<12)
//			sprintf(cmd,"S.2R%04ld\r",delay); // minimalnie moze byc .2 min.
//		sprintf(cmd,"S.%dR%04ld\r",UPS_SHUTDOWN_TIME/6,delay);
//	}
	if(m_Info->GetTurnOffDelay() >= 60)
		sprintf(cmd,"S%02dR%04ld\r", m_Info->GetTurnOffDelay()/60, delay);
	else
	{
		if(m_Info->GetTurnOffDelay() < 12)
			sprintf(cmd,"S.2R%04ld\r",delay); // minimalnie moze byc .2 min.
		else
			sprintf(cmd,"S.%dR%04ld\r", m_Info->GetTurnOffDelay()/6, delay);
	}
	//Z0006, LT - koniec

	// wysylamy rozkaz (nie ma on potwierdzenia)
	//SendPacket(cmd);
	SendPacket(cmd);
	usleep(1200000);
	SendPacket(cmd);
	usleep(1200000);
	SendPacket(cmd);
	usleep(1200000);
	return TRUE;
} // CUPSSaturn::DoRestart


/////////////////////////////////////////////////////////////////////////////
// wysyla (z ewentualna retransmisja) pakiet Cmd i czeka na odpowiedz
// zwraca TRUE gdy odebrano odpowiedz
// zwraca FALSE gdy wystapil timeout, lub m_Info->m_ActionType!=ACTION_NO_BREAK
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::SendAndGetResponse(LPCTSTR Cmd, BYTE *Buff, int &Len)
{
	BYTE action;
	int retry=0;
	while(++retry<=RESPONSE_RETRY)
	{
		SendPacket(Cmd);
		if(GetPacket(Buff,Len,Cmd))
		{
			// sprawdzamy czy to nie jest echo (echo oznacza blad)
			if(((int)strlen(Cmd)!=Len) || strncmp((char*)Buff,Cmd,Len))
				return TRUE;
		}
		if(CheckAction(action))
			return FALSE;
	}
	return FALSE;
} // CUPSSaturn::SendAndGetResponse


/////////////////////////////////////////////////////////////////////////////
void CUPSSaturn::SendPacket(LPCTSTR Cmd)
{
	m_ComPort.WriteStr((BYTE*)Cmd,strlen(Cmd));
} // CUPSSaturn::SendPacket


/////////////////////////////////////////////////////////////////////////////
// zwraca TRUE gdy odebrano pakiet
// zwraca FALSE gdy wystapil timeout, lub m_Info->m_ActionType!=ACTION_NO_BREAK
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::GetPacket(BYTE *Buff, int &Len, LPCTSTR Cmd)
{
	int ch, pos=0;
	BYTE action;
	time_t tm;
	time_t start=time(&tm);
	while((time(&tm)-start)<RESPONSE_TIMEOUT)
	{
		while((ch=m_ComPort.ReadChar())!=-1)
		{
			if(ch=='(' || ch=='#')
				pos=0;
			if(pos>=Len)
				continue;
			Buff[pos++]=ch;
			if(CheckPacket(Buff,pos,Cmd))
			{
				Buff[pos]='\0';
				Len=pos;
				return TRUE;
			}
		}
		if(CheckAction(action))
			return FALSE;
		usleep(100000);
	}
	return FALSE;
} // CUPSSaturn::GetPacket


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSaturn::CheckPacket(BYTE *Buff, int Len, LPCTSTR Cmd)
{
	if(Len<2)
		return FALSE;
	if(Buff[Len-1]!='\r')
		return FALSE;
	// sprawdzamy dokladny format (uzywamy tylko rozkazow I oraz Q1)
	if(!strcmp(Cmd,"I\r"))
	{
		if(Buff[0]!='#')
			return FALSE;
		return TRUE;
	}
	if(!strcmp(Cmd,"Q1\r"))
	{
		if(Buff[0]!='(')
			return FALSE;
		return TRUE;
	}
	return FALSE;
} // CUPSSaturn::CheckPacket


/////////////////////////////////////////////////////////////////////////////
long CUPSSaturn::GetNum(BYTE *Buff, int &Offset)
{
	char *ptr;
	long num=strtol((char*)Buff+Offset,&ptr,10);
	Offset=(ptr-(char*)Buff)+1;
	return num;
} // CUPSSaturn::GetNum

// EOF //////////////////////////////////////////////////////////////////////
