///////////////////////////////////////////////////////
// Fideltronik R&D Department, Copyright (c)2003
//         $Author: ltokarski $
//         $Date: 2004/07/20 10:18:55 $
//         $Locker:  $
//         $Revision: 1.9 $
///////////////////////////////////////////////////////
//	Opis funkcjonalny moduu:
//          Komunikacja przez protok Smartmon
///////////////////////////////////////////////////////
// Historia zmian:
// $Log: upssmart.cpp,v $
// Revision 1.9  2004/07/20 10:18:55  ltokarski
// Z0013 - wypisywanie w statusie protokolu komunikacji
//
// Revision 1.8  2004/07/09 11:32:01  ltokarski
// Z0012 - zapis biezacego statusu do pliku
//
// Revision 1.7  2004/07/01 10:10:59  ltokarski
// Z0010 - poprawienie wypisywania producenta i modelu
//
// Revision 1.6  2004/06/24 12:20:08  ltokarski
// Z0006 - konfigurowany czas opoznienia do wylaczenia (oprocz stykowych)
//
// Revision 1.5  2004/06/09 11:32:03  ltokarski
// nagwki CVS
// standard plikw Unix
//
///////////////////////////////////////////////////////

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <pthread.h>
#include <signal.h>
#include <termios.h>
#include <string.h>
#include <ctype.h>
#include "linux_ty.h"
#include "comport.h"
#include "upsmon.h"
#include "ups.h"
#include "upssmart.h"
#include "thread.h"

#define COM_CONTROL_STRING   "1200,N,8,1"
#define MAX_UPS_PACKET_LEN   133  // maksymalna dlugosc pakietow z UPSa
#define RESPONSE_TIMEOUT     2 // czas oczekiwania na pakiet z UPS'a
#define RESPONSE_RETRY       2    // ilosc retransmisji pakietu przed wyslaniem UPS_NO_COMMUNICATION


extern int __debug;

/////////////////////////////////////////////////////////////////////////////
CUPSSmartmon::CUPSSmartmon(CUPSThreadInfo *Info) : CUPSGeneric(Info)
{
	m_Protocol = 2;		//Z0013, LT
} // CUPSSmartmon::CUPS


/////////////////////////////////////////////////////////////////////////////
CUPSSmartmon::~CUPSSmartmon()
{
} // CUPSSmartmon::~CUPS

/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::OpenUPS()
{
//	m_Info->m_CS.Lock();
//	int port=m_Info->m_RSPort;
//	m_Info->m_CS.Unlock();
//	if(!m_ComPort.Open(port,COM_CONTROL_STRING))
	if(!m_ComPort.Open(m_Info->m_RSPort, COM_CONTROL_STRING))
		return FALSE;
	if(!m_ComPort.IsOpen())
		return FALSE;
	return TRUE;
} // CUPSSmartmon::OpenUPS

/////////////////////////////////////////////////////////////////////////////
void CUPSSmartmon::CloseUPS()
{
	// ustawiamy takie sygnaly zeby ewentualnie podlaczony UPS z interfejsem
	// stykowym nie wylaczyl sie
	m_ComPort.SetState(0,1);
	m_ComPort.Close();
} // CUPSSmartmon::CloseUPS


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::DoSetAutostart(BOOL Autostart)
{
	BYTE buff[MAX_UPS_PACKET_LEN+1];
	int buffLen=sizeof(buff);

	if(__debug)
		printf("Smart AutoStart=%d ava=%d\n",Autostart,m_AvailPar[17]);

	if(m_AvailPar[17]) // ATR
	{
		BOOL retVal=TRUE;
		buffLen=sizeof(buff);
		if(Autostart)
			retVal=SendAndGetResponse("^S004ATR1",buff,buffLen); // 1 - Automatic Restart
		else
			retVal=SendAndGetResponse("^S004ATR2",buff,buffLen); // 2 - Manual Restart
		if(!retVal)
			return FALSE;
		if(strcmp((char*)buff,"^1"))
			return FALSE;
		if(__debug)
			printf("Smart AutoStart OK\n");
		return TRUE;
	}
	return FALSE;
} // CUPSSmartmon::DoSetAutostart


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::DoTurnOff(BOOL SetAutostart,BYTE)
{
	if(__debug)
		printf("Turn OFF \n");

	if(!DoSetAutostart(SetAutostart))
	{
		if(__debug)
			printf("DoSetAuto FALSE\n");
		return FALSE;
	}
	BYTE buff[MAX_UPS_PACKET_LEN+1];
	int buffLen=sizeof(buff);

  // jezeli dostepny jest rozkaz SDA, to ustawiamy SDA2 (Entire UPS)
	if(m_AvailPar[82]) // SDA
	{
		if(__debug)
			printf("Ava_82\n");
		buffLen=sizeof(buff);
		if(!SendAndGetResponse("^S004SDA2",buff,buffLen))
			return FALSE; // 2 - Entire UPS
		if(strcmp((char*)buff,"^1"))
			return FALSE;
	}

	if(m_AvailPar[83]) // PSD
	{
		if(__debug)
			printf("Ava_83\n");
		buffLen=sizeof(buff);
		char sendBuff[20];
		//Z0006, LT - teraz czas jest zmienna
//		int len=sprintf(sendBuff,"%d",UPS_SHUTDOWN_TIME); // okreslamy dlugosc rozkazu PSD
//		sprintf(sendBuff,"^S%03dPSD%d",len+3,UPS_SHUTDOWN_TIME);
		int len=sprintf(sendBuff, "%d", m_Info->GetTurnOffDelay()); // okreslamy dlugosc rozkazu PSD
		sprintf(sendBuff, "^S%03dPSD%d", len+3, m_Info->GetTurnOffDelay());
		//Z0006, LT - koniec
		if(!SendAndGetResponse(sendBuff,buff,buffLen))
			return FALSE;
		if(strcmp((char*)buff,"^1"))
			return FALSE;
		if(__debug)
			printf("Ava_83 %s OK\n",sendBuff);
		return TRUE;
	}
	return FALSE;
} // CUPSSmartmon::DoTurnOff //^S004PSD2


/////////////////////////////////////delay////////////////////////////////////////
BOOL CUPSSmartmon::DoRestart()
{
	if(__debug)
		printf("Do Reastart Smartmonn\n");
	m_Info->m_CS.Lock();
	long delay=m_Info->m_ActionData; // opoznienie (w sek.) ponownego zalaczenia UPS'a
	if(__debug)
		printf("Wlaczenie po delay=%ld\n",m_Info->m_ActionData);
	m_Info->m_CS.Unlock();

	BYTE buff[MAX_UPS_PACKET_LEN+1];
	int buffLen=sizeof(buff);

	// jezeli dostepny jest rozkaz ATR, to ustawiamy ATR2 (Manual Restart)
	if(m_AvailPar[17]) // ATR
	{
		if(__debug)
			printf("Atr 17\n");
		buffLen=sizeof(buff);
		if(!SendAndGetResponse("^S004ATR2",buff,buffLen))
			return FALSE;
		if(strcmp((char*)buff,"^1"))
		{
			if(__debug)
				printf("Atr 17 %s\n",buff);
			return FALSE;
		}
	}

	// jezeli dostepny jest rozkaz SDA, to ustawiamy SDA1 (Output only)
	if(m_AvailPar[82]) // SDA
	{
		if(__debug)
			printf("Atr 82\n");
		buffLen=sizeof(buff);
		if(!SendAndGetResponse("^S004SDA1",buff,buffLen))
			return FALSE; // 2 - Entire UPS
		if(strcmp((char*)buff,"^1"))
		{
			if(__debug)
				printf("Atr 82 %s\n",buff);
			return FALSE;
		}
	}

	if(m_AvailPar[83]) // PSD
	{
		if(__debug)
			printf("Atr 83\n");
		buffLen=sizeof(buff);
		char sendBuff[20];
		//Z0006, LT - teraz czas jest zmienna
//		int len=sprintf(sendBuff,"%d",UPS_SHUTDOWN_TIME); // okreslamy dlugosc rozkazu PSD
//		sprintf(sendBuff,"^S%03dPSD%d",len+3,UPS_SHUTDOWN_TIME);
		int len=sprintf(sendBuff, "%d", m_Info->GetTurnOffDelay()); // okreslamy dlugosc rozkazu PSD
		sprintf(sendBuff, "^S%03dPSD%d", len+3, m_Info->GetTurnOffDelay());
		//Z0006, LT - koniec
		if(!SendAndGetResponse(sendBuff,buff,buffLen))
			return FALSE; //^S006PSD002
		if(strcmp((char*)buff,"^1"))
		{
			if(__debug)
				printf("Atr 83 %s\n",buff);
			return FALSE;
		}
	}
	else
		return FALSE;

	if(m_AvailPar[85]) // STD
	{
		if(__debug)
			printf("Atr 85\n");
		buffLen=sizeof(buff);
		char sendBuff[20];
		int len=sprintf(sendBuff,"%ld",delay); // okreslamy dlugosc rozkazu STD
		sprintf(sendBuff,"^S%03dSTD%ld",len+3,delay);
		if(!SendAndGetResponse(sendBuff,buff,buffLen))
			return FALSE; //^S006dSTD002
		if(strcmp((char*)buff,"^1"))
		{
			if(__debug)
				printf("Atr 85 %s\n",buff);
			return FALSE;
		}
		return TRUE;
	}

	return FALSE;

} // CUPSSmartmon::DoRestart




/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::GetStaticData()
{
	BYTE buff[MAX_UPS_PACKET_LEN+1];
	int buffLen=sizeof(buff);

	if(SendAndGetResponse("^P003AP1",buff,buffLen))
	{
		if(buffLen<5)
			return FALSE;
		int offset=5; // ustawiamy offset po ^D(LEN)
		int num;
		while(offset<buffLen)
		{
			num=GetNum(buff,offset);
			m_AvailPar[num]=1;
//printf("m_Ava %d\n",num);
		}
	}
	else
		return FALSE;

	buffLen=sizeof(buff);
	if(SendAndGetResponse("^P003AP2",buff,buffLen))
	{
		if(buffLen<5)
			return FALSE;
		int offset=5; // ustawiamy offset po ^D(LEN)
		int num;
		while(offset<buffLen)
		{
			num=GetNum(buff,offset);
			m_AvailPar[num]=1;
		}
	}
	else
		return FALSE;


	if(m_AvailPar[55])
	{
		buffLen=sizeof(buff);
		if(SendAndGetResponse("^P003MAN",buff,buffLen))
		{
			//printf("Nazwa urzadzenia %s %d\n",buff,buffLen);
			m_Info->m_CS.Lock();
			memset(m_Info->m_Manufacturer,0,sizeof(m_Info->m_Manufacturer));
			//Z0010, LT - blad, brak sprawdzenia, czy string sie zmiesci. Rowniez wycina polskie znaki
//			if(buffLen>5)
//				 memcpy(m_Info->m_Manufacturer,buff+5,buffLen-5);
//			else
//				buffLen=5;   // na potrzeby if(buffLen-5<...)
//			if(buffLen-5<LEN_MAN)
//				memset(m_Info->m_Manufacturer+(buffLen-5),' ',LEN_MAN-(buffLen-5));
//			for(int i=0; i<LEN_MAN; i++)
//				if(!isalnum(m_Info->m_Manufacturer[i]))
//					m_Info->m_Manufacturer[i]=' ';
			if(buffLen - 5 >= LEN_MAN)
			{
				strncpy(m_Info->m_Manufacturer, (char*)buff+5, LEN_MAN-1);
				m_Info->m_Manufacturer[LEN_MAN] = 0;
			}
			else
				strcpy(m_Info->m_Manufacturer, (char*)buff+5);
			//Z0010, LT - koniec
			m_Info->m_CS.Unlock();
			//printf("Nazwa urzadzenia %s %d\n",m_Info->m_Manufacturer,buffLen);
			char d[LEN_MAN];
			memset(d,0,sizeof(d));
			strncpy(d,(char *)m_Info->m_Manufacturer,LEN_MAN);
//			printf("+ Manufacture %s\n",d);
			printf("+ Producent: %s\n",d);
		}
		else
			return FALSE;
	}
	else
	{
		m_Info->m_CS.Lock();
		memset(m_Info->m_Manufacturer,0xFF,LEN_MAN);
		m_Info->m_CS.Unlock();
	}

	if(m_AvailPar[56])
	{
		buffLen=sizeof(buff);
		if(SendAndGetResponse("^P003MOD",buff,buffLen))
		{
			m_Info->m_CS.Lock();
			//Z0010, LT - blad, brak sprawdzenia, czy string sie zmiesci. Rowniez wycina polskie znaki
//			if(buffLen>5)
//				memcpy(m_Info->m_Model,buff+5,buffLen-5);
//			else
//				buffLen=5;   // na potrzeby if(buffLen-5<...)
//			if(buffLen-5<LEN_MODEL)
//				memset(m_Info->m_Model+(buffLen-5),' ',LEN_MODEL-(buffLen-5));
//			for(int i=0; i<LEN_MODEL; i++)
//				if(!isalnum(m_Info->m_Model[i]))
//					m_Info->m_Model[i]=' ';
			if(buffLen - 5 >= LEN_MODEL)
			{
				strncpy(m_Info->m_Model, (char*)buff+5, LEN_MODEL-1);
				m_Info->m_Model[LEN_MODEL] = 0;
			}
			else
				strcpy(m_Info->m_Model, (char*)buff+5);
			//Z0010, LT - koniec
			m_Info->m_CS.Unlock();
			char d[LEN_MODEL];
			memset(d,0,sizeof(d));
			strncpy(d, (char *)m_Info->m_Model, LEN_MODEL);	//Z0010, LT - bylo: LEN_MAN
			printf("+ Model: %s\n",d);
		}
		else
			return FALSE;
	}
	else
	{
		m_Info->m_CS.Lock();
		memset(m_Info->m_Model,0xFF,LEN_MODEL);
		m_Info->m_CS.Unlock();
	}
	if(m_AvailPar[84])
	{
		buffLen=sizeof(buff);
		if(SendAndGetResponse("^P003VER",buff,buffLen))
		{
			m_Info->m_CS.Lock();
			//Z0012, LT - wszystko zle! nie sprawdza, czy sie zmiesci, niepotrzebnie dopelnia spacjami,
			//a potem likwidujac niedrukowalne likwiduje \0
//			if(buffLen>5)
//				memcpy(m_Info->m_SoftVer,buff+5,buffLen-5);
//			else
//				buffLen=5;   // na potrzeby if(buffLen-5<...)
//			if(buffLen-5<LEN_SOFT_VER)
//				memset(m_Info->m_SoftVer+(buffLen-5),' ',LEN_SOFT_VER-(buffLen-5));
//			for(int i=0; i<LEN_SOFT_VER; i++)
//				if(!isprint(m_Info->m_SoftVer[i]))
//					m_Info->m_SoftVer[i]=' ';
			buffLen -= 5; //obcinam poczatek pakietu
			if(buffLen < 0)		//np kod bledu
				m_Info->m_SoftVer[0] = 0;
			else
			{
				strncpy(m_Info->m_SoftVer, (char*)buff+5, LEN_SOFT_VER);
				m_Info->m_SoftVer[(buffLen < LEN_SOFT_VER)? buffLen: LEN_SOFT_VER-1] = 0;
			}
			//Z0012, LT - koniec
			m_Info->m_CS.Unlock();
		}
		else
			return FALSE;
	}
	else
	{
		m_Info->m_CS.Lock();
		memset(m_Info->m_SoftVer,0xFF,LEN_SOFT_VER);
		m_Info->m_CS.Unlock();
	}
	m_Info->m_CS.Lock();
	m_Info->m_CanAutostart=m_AvailPar[17]; // 17 - ATR
	m_Info->m_CanTurnOff=m_AvailPar[83];   // 83 - PSD
	m_Info->m_AvpMan=m_AvailPar[55];       // 55 - Manufacturer
	m_Info->m_AvpMod=m_AvailPar[56];       // 56 - Model
	m_Info->m_AvpSoftVer=m_AvailPar[84];   // 84 - Software Version
	m_Info->m_BatAutonomy=-1;
	m_Info->m_CS.Unlock();
	 return TRUE;
} // CUPSSmartmon::GetStaticData


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::GetDynamicData()
{
	//printf("GetDynamicData %d\n",m_UPSType);
	BYTE buff[MAX_UPS_PACKET_LEN+1];
	int offset, buffLen=sizeof(buff);
	char batCondition=-1;         // 19 - Battery Condition
	char batStatus=-1;            // 22 - Battery Status
	short batVol=-1;              // 24 - Battery Voltage
	short inVol=-1;               // 51 - Input Voltage 1
	short outCurrent=-1;          // 65 - Output Current 1
	short outLoad=-1;             // 69 - Output Load 1
	long outPower=-1;             // 73 - Output Power 1
	char outSource=-1;            // 76 - Output Source
	short outVol=-1;              // 77 - Output Voltage 1

	if(m_AvailPar[19] || m_AvailPar[22] || m_AvailPar[24])
	{
		if(SendAndGetResponse("^P003ST1",buff,buffLen))
		{
			//printf("SendAndGetResponce %s\n",buff);
			if(buffLen<5)
				return FALSE;
			offset=5; // ustawiamy offset po ^D(LEN)
			if(offset<buffLen)
				batCondition=(char)GetNum(buff,offset); // Battery Condition
			if(m_AvailPar[19]==0)
				batCondition=-1;
			if(offset<buffLen)
				batStatus=(char)GetNum(buff,offset); // Battery Status
			if(m_AvailPar[22]==0)
				batStatus=-1;
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Battery Charge
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Seconds on Battery
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Estimated Minutes
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Estimated Charge
			if(offset<buffLen)
				batVol=(short)GetNum(buff,offset); // Battery Voltage
			if(m_AvailPar[24]==0)
				batVol=-1;
		}
		else
			return FALSE;
	}

	if(m_AvailPar[51])
	{
		buffLen=sizeof(buff);
		if(SendAndGetResponse("^P003ST2",buff,buffLen))
		{
			//printf("send 1 %s\n",buff);
			if(buffLen<5)
				return FALSE;
			offset=5; // ustawiamy offset po ^D(LEN)
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Input Line Bads
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Input Num Lines
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Input Frequency 1
			if(offset<buffLen)
				inVol=(short)GetNum(buff,offset); // Input Voltage 1
			if(m_AvailPar[51]==0)
				inVol=-1;
			//printf("inVol=%d\n",inVol);
		}
		else
			return FALSE;
	}

	if(m_AvailPar[65] || m_AvailPar[69] || m_AvailPar[73] || m_AvailPar[76] || m_AvailPar[77])
	{
		buffLen=sizeof(buff);
		if(SendAndGetResponse("^P003ST3",buff,buffLen))
		{
			//printf("Send %s\n",buff);
			if(buffLen<5)
				return FALSE;
			offset=5; // ustawiamy offset po ^D(LEN)
			if(offset<buffLen)
				outSource=(char)GetNum(buff,offset); // Output Source
			if(m_AvailPar[76]==0)
				outSource=-1;
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Output Frequency
			if(offset<buffLen)
				GetNum(buff,offset);   // pomijamy Output Num Lines
			if(offset<buffLen)
				outVol=(short)GetNum(buff,offset); // Output Voltage 1
			if(m_AvailPar[77]==0)
				outVol=-1;
			if(offset<buffLen)
				outCurrent=(short)GetNum(buff,offset); // Output Current 1
			if(m_AvailPar[65]==0)
				outCurrent=-1;
			if(offset<buffLen)
				outPower=GetNum(buff,offset); // Output Power 1
			if(m_AvailPar[73]==0)
				outPower=-1;
			if(offset<buffLen)
				outLoad=(short)GetNum(buff,offset); // Output Load 1
			if(m_AvailPar[69]==0)
				outLoad=-1;
		}
		else
			return FALSE;
	}

	m_Info->m_CS.Lock();
	m_Info->m_BatCondition=batCondition;
	m_Info->m_BatStatus=batStatus;
	m_Info->m_BatVol=batVol;
	m_Info->m_InVol=inVol;
	m_Info->m_OutCurrent=outCurrent;
	m_Info->m_OutLoad=outLoad;
	m_Info->m_OutPower=outPower;
	m_Info->m_OutSource=outSource;
	m_Info->m_OutVol=outVol;
	m_Info->m_CS.Unlock();
	m_BatStatus=batStatus;
	m_OutSource=outSource;
	return TRUE;
} // CUPSSmartmon::GetDynamicData

/////////////////////////////////////////////////////////////////////////////
// wysyla (z ewentualna retransmisja) pakiet Cmd i czeka na odpowiedz
// zwraca TRUE gdy odebrano odpowiedz
// zwraca FALSE gdy wystapil timeout, lub m_Info->m_Terminate==TRUE
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::SendAndGetResponse(LPCTSTR Cmd, BYTE *Buff, int &Len)
{

	BYTE action;
	int retry=0;
	while(++retry<=RESPONSE_RETRY)
	{
//printf("Send %s \n",Cmd);
		SendPacket(Cmd);
		if(GetPacket(Buff,Len))
		{
			//printf("Get %s\n",Buff);
		 	return TRUE;
		}
		if(CheckAction(action))
			return FALSE;
	}
	//printf("Get ERROR \n");
	return FALSE;
} // CUPSSmartmon::SendAndGetResponse


/////////////////////////////////////////////////////////////////////////////
void CUPSSmartmon::SendPacket(LPCTSTR Cmd)
{
	m_ComPort.WriteStr((BYTE*)Cmd,strlen(Cmd));
} // CUPSSmartmon::SendPacket


/////////////////////////////////////////////////////////////////////////////
// zwraca TRUE gdy odebrano pakiet
// zwraca FALSE gdy wystapil timeout, lub m_Info->m_Terminate==TRUE
/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::GetPacket(BYTE *Buff, int &Len)
{
	int ch, pos=0;
	BYTE action;
	time_t tm;
	time_t start=time(&tm);


	//printf("Get Pac %d timeout=%d\n",start,RESPONSE_TIMEOUT);
	while((time(&tm)-start)<RESPONSE_TIMEOUT)
	{
//printf("Get Pac %d time=%d\n",time(&tm),time(&tm)-start);
		while((ch=m_ComPort.ReadChar())!=-1)
		{

			if(ch=='^')
				pos=0;
			if(pos>=Len)
				continue;
			Buff[pos++]=ch;
			if(CheckPacket(Buff,pos))
			{
				Buff[pos]='\0';
				Len=pos;
				//printf("TRYUE\n");
				return TRUE;
			}
		}
		if(CheckAction(action))
			return FALSE;
		usleep(100000);
	}
	//printf("Get packet timeout %ld %ld\n",time(&tm)-start,RESPONSE_TIMEOUT);
	return FALSE;
} // CUPSSmartmon::GetPacket


/////////////////////////////////////////////////////////////////////////////
BOOL CUPSSmartmon::CheckPacket(BYTE *Buff, int Len)
{
	if(Len<2)
		return FALSE;
	if(Buff[1]=='0' || Buff[1]=='1' || Buff[1]=='*')
		return TRUE;
	if(Buff[1]!='D')
		return FALSE;
	if(Len<5)
		return FALSE;
	char buff[4]={0};
	memcpy(buff,Buff+2,3);
	if(Len!=(5+atoi(buff)))
		return FALSE;
	return TRUE;
} // CUPSSmartmon::CheckPacket


/////////////////////////////////////////////////////////////////////////////
long CUPSSmartmon::GetNum(BYTE *Buff, int &Offset)
{
	char *ptr;
	long num=strtol((char*)Buff+Offset,&ptr,10);
	Offset=(ptr-(char*)Buff)+1;
	return num;
} // CUPSSmartmon::GetNum

